/* nyp2txt.c V012
Convert
 Navin'you format track data(DriveLog/* or .NYP)
to
 Waypoint+ text format TRACK data(.txt)
*/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <math.h>

#define byte unsigned char
#define word unsigned short int
#define dword unsigned long

#define ASCALE 0xB60B60	/* 0xfffffff / 360 32*7*17*3133 */

#define LATMIN (20*ASCALE)
#define LATMAX (48*ASCALE)
#define LONMIN (120*ASCALE)
#define LONMAX (150*ASCALE)

FILE *INFile;
long NYPEndPos;

#pragma pack(1)
/* directory descriptor */
typedef struct NYPDirDesc_t {
	word  Unknown;
	word  numSlot;
	dword  DirPosBigI;
	byte  Unknown2[24];
	char  IDString[12];
} NYPDirDesc;

typedef struct NYPDirEnt_t {
	word   Unknown[3];
	dword  SectionPosBigI;
	dword  SectionLenBigI;
	word   Unkonwn1;
} NYPDirEnt;

/* 26 byte record  */
typedef struct NYPRecord_t {
	dword  ILon;
	dword  ILat;
	dword  IAng;
	dword  Unknown3;
	dword  recTime;
	dword  Unknown5;
/*	word  Unknown51;*/
	word   Unknown6;
} NYPRecord;
#pragma pack()

NYPDirDesc DirDesc;
NYPDirEnt  DirEnt;
NYPRecord  record;
unsigned long dirPos,sectionPos,sectionLen;

byte dummy[0x100];

struct tm *tmp;

void usage(void);
void cvt(void);
void xfread( void *buffer, size_t size, size_t count, FILE *stream );
void xfseek( FILE *stream, long offset, int origin );
dword dswab(dword big);
word wswab(word big);

void main(int argc,char *argv[])
{
	fprintf(stderr,"NYP2WAY - Navin'You NYP/DriveLog file to Waypoint+ track format\n");
	fprintf(stderr,"26-Mar-98 gigo@yk.rim.or.jp|imaizumi@nisiq.net Ver.012\n");
	if (argc < 2)
		usage();
	if ((INFile = fopen(argv[1],"r+b")) == NULL) {
		printf("\n*Error - can't open NYP file -- '%s' !\n",argv[1]);
		exit(EXIT_FAILURE);
	}
	printf("Datum,Tokyo mean,Bessel 1841,739.845,0.10037483,-128,481,664\n");
	if (strstr(strupr(argv[1]),".NYP") != NULL) {	/* NYP FILE */
		xfseek(INFile,0L,SEEK_END);
		NYPEndPos = ftell(INFile);
		xfseek(INFile,NYPEndPos - sizeof(DirDesc),SEEK_SET);
		xfread(&DirDesc,sizeof(DirDesc),1,INFile);
		if (strncmp(&DirDesc.IDString[0],"SONY ITC NYP",12)!=0) {
			printf("\n*Error - Unknown format file -- '%s' !\n",argv[1]);
			exit(EXIT_FAILURE);
		}
		dirPos = dswab(DirDesc.DirPosBigI);
		xfseek(INFile,dirPos,SEEK_SET);
		for (;;){
			xfread(&DirEnt,sizeof(DirEnt),1,INFile);
			sectionPos = dswab(DirEnt.SectionPosBigI);
			sectionLen = dswab(DirEnt.SectionLenBigI);
			if (sectionLen == 0L)
				break;
			if ((DirEnt.Unknown[1] != 0x3144)||(DirEnt.Unknown[2] != 0x2121))
				continue;
			dirPos = ftell(INFile);
			cvt();
			xfseek(INFile,dirPos,SEEK_SET);
		}
	} else {
		sectionPos = 0L;
		xfseek(INFile,0L,SEEK_END);
		sectionLen = ftell(INFile);
		cvt();
	}
	exit(EXIT_SUCCESS);
}
#define TTBUFLEN 128
#define MTBUFLEN 128

char ttbuf[TTBUFLEN];
char mtbuf[MTBUFLEN];

void cvt(void)
{
	int i,s,deg,min,sec;
	short int tsb,si;
	dword twb,fn,ttlen,mtlen,nRec,n;
	long t,t0;
	dword ILat,ILon,ILat0,ILon0;

#ifdef DEBUG
	printf("pos:0x%08lx, len:0x%08lx\n",sectionPos,sectionLen);
#endif
	xfseek(INFile,sectionPos,SEEK_SET);
	xfread(&tsb,sizeof(tsb),1,INFile);
	si = wswab(tsb);			/* si always 1 ? */
	xfread(&twb,sizeof(twb),1,INFile);
	fn = dswab(twb);		/* file serial ? */
	xfread(&twb,sizeof(twb),1,INFile);
	ttlen = dswab(twb);		/* title length */
	memset(ttbuf,0,TTBUFLEN);
	xfread(ttbuf,sizeof(char),(size_t)ttlen,INFile);
	for (i=11;i;--i) {		/* dummy read */
		xfread(&twb,sizeof(twb),1,INFile);
#ifdef DEBUG
		printf("%08lx:",dswab(twb));
#endif
	}
	xfread(&twb,1,1,INFile);	/* dummy read */
	xfread(&twb,sizeof(twb),1,INFile);
	mtlen = dswab(twb);		/* map title length */
	memset(mtbuf,0,MTBUFLEN);
	xfread(mtbuf,sizeof(char),(size_t)mtlen,INFile);
	xfread(&twb,sizeof(twb),1,INFile);
	nRec = dswab(twb);		/* number of records */
	fprintf(stderr,"%s(%ld records) on %s\n",ttbuf,nRec,mtbuf);

	s = 1;	/* start */
	t0 = 0;
	for(n=1;n<=nRec;++n) {
		xfread(&record,sizeof(record),1,INFile);
		t = dswab(record.recTime)+(9*3600);
		if ( ((t-t0)<=0) || ((tmp = gmtime(&t))==NULL) ){
			fprintf(stderr,"Warnning - illegual time value skipped. (record %ld)\n",n);
			s = 1;	/* mark next record */
			continue;
		}
		if ((t - t0)> 3)	/* data interrupted ? */
			s = 1;
/*		TP,DMS, 35.244851624, 133.1939403690,03/21/1998,03:40:51,1*/
		ILat = dswab(record.ILat);
		ILon = dswab(record.ILon);

		if ( (ILat < LATMIN) || (ILat > LATMAX)
			|| ((n > 1) && ((labs(ILat-ILat0)/(t-t0)) > 0x1a300)) ) {
			fprintf(stderr,"Warnning - illegual Lat value skipped. (record %ld)\n",n);
			s = 1;	/* mark next record */
			continue;
		}
		if ( (ILon < LONMIN) || (ILon > LONMAX)
			|| ((n > 1) && ((labs(ILon-ILon0)/(t-t0)) > 0x1a300)) ) {
			fprintf(stderr,"Warnning - illegual Lon value skipped.(record %ld)\n",n);
			s = 1;	/* mark next record */
			continue;
		}

#ifdef DEBUG
/*		printf("2:%8.1f, 3:%8.1f, 5:%8.1f, 6:%04x\n",
			(float)dswab(record.Unknown2),
			(float)dswab(record.Unknown3),
			(float)dswab(record.Unknown5),
			record.Unknown6);*/
/*		printf("31:%04lx, 50:%04lx,51:%04lx, 6:%04x\n",*/
		printf("3:%08lx\n",
/*			wswab(record.Unknown30),*/
/*			wswab(record.Unknown31)/0x400,*/
			dswab(record.Unknown3));
/*			wswab(record.Unknown51),*/
/*			wswab(record.Unknown6));*/
#endif
		t0 = t;
		ILat0 = ILat;
		ILon0 = ILon;
		printf("TP,DMS, ");
#ifndef USE_FLOAT
		deg = (int)(ILat/ASCALE);
		ILat = (ILat%ASCALE)*60;
		min = (int)(ILat/ASCALE);
		ILat = (ILat%ASCALE)*60;
		sec = (int)(ILat/ASCALE);
		ILat = ((ILat%ASCALE)/3133)*100000;
		printf("%2d.%02d%02d%05ld, ",deg,min,sec,(long)(ILat/(ASCALE/3133)));
#else
		{
			double lat,x;

			lat = (float)dswab(record.ILat)/ASCALE;
			deg = (int)lat;
			x = lat - floor(lat);
			min = (int)(x = x*60);
			x = (x-floor(x))*60000000;
			sec = (long)x;
			printf("%2d.%02d%07ld, ",deg,min,sec);
		}
#endif
#ifndef USE_FLOAT
		deg = (int)(ILon/ASCALE);
		ILon = (ILon%ASCALE)*60;
		min = (int)(ILon/ASCALE);
		ILon = (ILon%ASCALE)*60;
		sec = (int)(ILon/ASCALE);
		ILon = ((ILon%ASCALE)/3133)*1000000;
		printf("%2d.%02d%02d%06ld,",deg,min,sec,(long)(ILon/(ASCALE/3133)));
#else
		{
			double lon,x;

			lon = (float)dswab(record.ILon)/ASCALE;
			deg = (int)lon;
			x = lon - floor(lon);
			min = (int)(x = x*60);
			x = (x-floor(x))*60000000;
			sec = (long)x;
			printf("%2d.%02d%08ld,",deg,min,sec);
		}
#endif
		printf("%02d/%02d/%04d,%02d:%02d:%02d,",tmp->tm_mon,tmp->tm_mday,tmp->tm_year+1900,tmp->tm_hour,tmp->tm_min,tmp->tm_sec);
		printf("%d\n",(s==1) ? s-- : 0);
		}
}

dword dswab(dword big)
{
	dword little;
	char *p,*q;

	p =((char *)&big) +3;
	q =(char *)&little;

	*q++ = *p--;
	*q++ = *p--;
	*q++ = *p--;
	*q = *p;
	return little;
}

word wswab(word big)
{
	word little;
	char *p,*q;

	p =((char *)&big)+1;
	q =(char *)&little;

	*q++ = *p--;
	*q = *p;
	return little;
}

void xfread( void *buffer, size_t size, size_t count, FILE *stream )
{
	if (fread(buffer, size, count, stream)!= count) {
		fprintf(stderr,"\nError - can't read.\n\n");
		exit(EXIT_FAILURE);
	}
}

void xfseek( FILE *stream, long offset, int origin )
{
	if (fseek(stream, offset, origin ) != 0) {
		fprintf(stderr,"\nError - can't seek.\n\n");
		exit(EXIT_FAILURE);
	}
}

void usage(void)
{
	fprintf(stderr,"\nUsage: nyp2way NYPFile\n");
}
