/* txt2log.c V0.12 by O.Imaizumi
Convert
 Waypoint+ .txt format TRACK data
  Datum:Tokyo
  Modes:DMS
to
 AtlasRD .log format

This program works only in the range of the
data which can be acquired in japanese country.
And it supposed input data's min and sec is under 60.
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

void cnvAngFmt(char *buf,double angle);
char *strtokX(char *str1,char *delim);

char lbuf[96],latbuf[12],lngbuf[12];

struct tm atime;
time_t jptime;
char tmbuf[32];

void main()
{
    char *p;

/*
Datum,WGS84,WGS84,0,0,0,0,0
Datum,Tokyo mean,Bessel 1841,739.845,0.10037483,-128,481,664
*/
    /* search & check Datum */
    while((p=gets(lbuf))!=NULL)
        if (strncmp(p,"Datum",5) == 0)
            break;
    if (p == NULL) {
        fprintf(stderr,"*Error - Unexpected EOF.\n");
        exit(1);
    }
    if (strncmp(p+6,"Tokyo mean",10)!=0 ) {
        fprintf(stderr,"*Error - illegal Datum('Tokyo mean' expected).\n");
        exit(1);
    }
/*
012345678901234567890123456789012345678901234567890123456789
TP,D, 34.757604003, 137.7385836840,07/05/1997,09:33:04,1
TP,DM, 34.454562402, 137.4431502104,07/05/1997,09:33:04,1
TP,DMS, 36.014155393, 138.5920911510,07/04/1997,17:41:21,0
*/
    /* search TP line & check modes */
    while((p=gets(lbuf))!=NULL)
        if (strncmp(p,"TP",2) == 0)
            break;
    if (p == NULL) {
        fprintf(stderr,"*Error - Unexpected EOF.\n");
        exit(1);
    }
    if (strncmp(p+3,"DMS",3) != 0) {
        fprintf(stderr,"*Error - illegal modes('DMS' expected).\n");
        exit(1);
    }

    tzset();
    do {
        if (strncmp(strtokX(lbuf,","),"TP",2) == 0) {
            if (strncmp(strtokX(NULL,","),"DMS",3) != 0) {
                fprintf(stderr,"*Error - illegal modes('DMS' expected).\n");
                exit(1);
            }
            cnvAngFmt(latbuf,atof(strtokX(NULL,",")));
            cnvAngFmt(lngbuf,atof(strtokX(NULL,",")));
            atime.tm_mon = atoi(strtokX(NULL,"/"))-1;
            atime.tm_mday = atoi(strtokX(NULL,"/"));
            atime.tm_year = atoi(strtokX(NULL,","))-1900;
            atime.tm_hour = atoi(strtokX(NULL,":"));
            atime.tm_min = atoi(strtokX(NULL,":"));
            atime.tm_sec = atoi(strtokX(NULL,","));
            jptime = mktime(&atime)+(9*3600L);

            strftime(tmbuf,32,"%y/%m/%d %H:%M:%S",localtime(&jptime));
            printf("%s N%s E%s\n",tmbuf,latbuf,lngbuf);
        }
    } while((p=gets(lbuf))!=NULL) ;
}

static char dbuf[12];

void cnvAngFmt(char *buf,double angle)
{
    char *p,*q;
    int deg,min;

    p = dbuf; q = buf;
    sprintf(dbuf,"%6.5f",angle); /* rounding off */

    for(;*p!='.';*q++ = *p++);
    *q++ = ':' ; p++;
    *q++ = *p++;*q++ = *p++;
    *q++ = ':' ; 
    *q++ = *p++;*q++ = *p++;
    *q++ = '.' ; 
    *q++ = *p++; *q = '\0';
    p = strrchr(buf,':');
    if (*++p != '6')
        return;
    /* carry */
    min = atoi(strchr(buf,':')+1)+1;
    deg = atoi(buf);
    if (min >= 60) {
        min = 0;
        deg++;
    }
    sprintf(buf,"%d:%02d:00.0",deg,min);
}

char *strtokX(char *str1,char *delim)
{
    char *p;

    if ((p = strtok(str1,delim))!=NULL)
        return(p);
    fprintf(stderr,"*Error - illegal line format.\n");
    exit(1);
}
