/* txt2trk.c V0.10
Convert
 Waypoint+ text format TRACK data(.txt)
to
 PCX5 format TRACK data(.trk) for kashmir 
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

double lat,lng;

struct tm atime;
time_t uxtime;
char tmbuf[32];

char *typstr[]= {
    "DEG","DM","DMS"
};

char lbuf[96];

void main()
{
    char *p;
    int typ;

    while((p=gets(lbuf))!=NULL)
        if (strncmp(p,"Datum",5) == 0)
            break;

    if (p == NULL) {
        printf("*Error - Unexpected EOF.\n");
        exit(1);
    }

    printf("\n");
    printf("H  SOFTWARE NAME & VERSION\n");
    printf("I  PCX5 2.08\n");
    printf("\n");
    printf("H  R DATUM                IDX DA            DF            DX            DY            DZ\n");
/*
Datum,WGS84,WGS84,0,0,0,0,0
Datum,Tokyo mean,Bessel 1841,739.845,0.10037483,-128,481,664
*/
    if (strncmp(p+6,"Tokyo",5)==0 ) {
        printf("M  G Tokyo                116 +7.398450e+02 +1.003748e-05 -1.280000e+02 +4.810000e+02 +6.640000e+02\n");
    } else if (strncmp(p+6,"WGS84",5)==0 ) {
        printf("M  G WGS 84               121 +0.000000e+00 +0.000000e+00 +0.000000e+00 +0.000000e+00 +0.000000e+00\n");
    } else {
        printf("*Error - illegal Datum(Tokyo or WGS 84 expected).\n");
        exit(1);
    }
    printf("\n");
    printf("H  COORDINATE SYSTEM\n");
/*
012345678901234567890123456789012345678901234567890123456789
TP,D, 34.757604003, 137.7385836840,07/05/1997,09:33:04,1
TP,DM, 34.454562402, 137.4431502104,07/05/1997,09:33:04,1
TP,DMS, 36.014155393, 138.5920911510,07/04/1997,17:41:21,0
*/
    while((p=gets(lbuf))!=NULL)
        if (strncmp(p,"TP",2) == 0)
            break;
    if (p == NULL) {
        printf("*Error - Unexpected EOF.\n");
        exit(1);
    }
    for (typ = 2; typ >= 0; --typ)
        if (strncmp(p+3,typstr[typ],typ+1) == 0)
            break;
    if (typ < 0) {
        printf("*Error - illegal format(D or DM or DMS expected).\n");
        exit(1);
    }

    printf("U  LAT LON %s\n",typstr[typ]);
    printf("\n");
    printf("H  LATITUDE    LONGITUDE    DATE      TIME     ALT    ;track\n");

    tzset();
    do {
        if (strncmp(p,"TP",2) == 0) {
            if (strncmp(p+3,typstr[typ],typ+1) != 0) {
                printf("*Error - illegal format(%s expected).\n",typstr[typ]);
                exit(1);
            }
            p += typ;   /* add offset.(typ == offset) */
            lat = atof(p+6);
            lng = atof(p+20);
            atime.tm_sec = atoi(p+52);
            atime.tm_min = atoi(p+49);
            atime.tm_hour = atoi(p+46);
            atime.tm_mon = atoi(p+35)-1;
            atime.tm_mday = atoi(p+38);
            atime.tm_year = atoi(p+41)-1900;
            uxtime = mktime(&atime)+(9*3600L);

            strftime(tmbuf,32,"%d-%b-%y %H:%M:%S",localtime(&uxtime));
/*
U  LAT LON DEG
T  N35.5406968 E139.4345175 08-JUL-97 14:44:17 -9999 
U  LAT LON DM
T  N3532.63909 E13925.88437 08-JUL-97 14:44:17 -9999 
U  LAT LON DMS
T  N344527.374 E1374418.901 05-JUL-97 09:33:04 -9999 
*/
            switch(typ) {
            case 0:
                printf("T  N%03.7f E%03.7f %s -9999 \n",lat,lng,tmbuf);
                break;
            case 1:
                printf("T  N%05.5f E%05.5f %s -9999 \n",lat*100.0,lng*100.0,tmbuf);
                break;
            case 2:
                printf("T  N%07.3f E%07.3f %s -9999 \n",lat*10000.0,lng*10000.0,tmbuf);
                break;
            }
        }
    } while((p=gets(lbuf))!=NULL) ;
}
